var Main = function(win)
{
    var self = this;
    var screenCompanion = null;
    var gameSimulatorToolkit = null;

    var isVuplexAvailable = null;
    
    var init = function()
    {
        // we are here because onDomLoaded has been triggered 
        // if we are in VR, the url contains ?waitVuplex=1 and we must wait that window.vuplex becomes available
        // on real PC/Mac webbrowser we don't need vuplex because we simulate VR communication using GameSimulatorToolkit
        initDelayed(document.location.href.indexOf("waitVuplex=1") >-1);
    }

    self.delayedCall = function(funcToCall, delayMs)
    {
        (async function() { 
            try
            {
                if(navigator.userAgent.indexOf('AnkaaHeadless')>-1)
                    await Task.Delay(delayMs); 
                else
                    await new Promise(r => setTimeout(r, delayMs));

                funcToCall();    
            }
            catch(e1)
            {
                console.log(e1.message);
                console.log(e1.stack);
            }
        })();
    }

    var initDelayed = function(waitForVuplex){
        var initialized = false;
        if(!waitForVuplex)
        {
            gameSimulatorToolkit = new GameSimulatorToolkit(function(jsonData) { onMessageReceived({'data': jsonData}); });
            screenCompanion = new ScreenCompanion(sendRawMessage);
            screenCompanion.zoomForDevelopement();
            initialized = true;
        }
        else
        {
            isVuplexAvailable = typeof(window.vuplex) != "undefined";
            if(isVuplexAvailable)
            {
                screenCompanion = new ScreenCompanion(sendRawMessage);
                window.vuplex.addEventListener('message', onMessageReceived);
                initialized = true;
            }
        }
        if(!initialized)
            setTimeout(function(){ initDelayed(waitForVuplex) }, 200); // retry in 200ms

    }
    
    var onMessageReceived = function(event)
    {
        try
        {
            let json = event.data;
            screenCompanion.processMessage(json);   
        }
        catch(e1)
        {
            console.log("CompanionScreen JS exception in Main:onMessageReceived: "+e1.message+"\n"+e1.stack);
        }
    }

    var sendRawMessage = function(json)
    {
        // if vuplex exists
        if(isVuplexAvailable)
        {
            window.vuplex.postMessage(json);
        }
        else
        {
            gameSimulatorToolkit.companionOutgoingMessage(json);
        }
    }
    
    init.call(self);
}

