var ScreenCompanion = function(sendRawMessageAction)
{
    var self = this;
    var containerDiv = null;

    var lastClickId = 0;

    var screenCompanionTabs = null;
    var screenCompanionGame = null;
    var screenCompanionAccount = null;
    var screenCompanionInventory = null;
    var screenCompanionTeammates = null;
    var screenCompanionSettings = null;
    var screenCompanionHelp = null;
    var screenCompanionDebugConsole = null;
    var screenCompanionDevTools = null;

    var moneyInfoDiv = null;

    var init = function()
    {
        containerDiv = document.getElementById("screenCompanionContainer");

        containerDiv.addEventListener('click', (event) => { 
            drawClickAt(event.clientX, event.clientY); 
        });

        moneyInfoDiv = containerDiv.querySelector("#scMoneyInfo");
        moneyInfoDiv.textContent = "¥ 123 456 789"

        screenCompanionGame = new ScreenCompanionGame(self.sendMessage);
        screenCompanionAccount = new ScreenCompanionAccount(self.sendMessage);
        screenCompanionInventory = new ScreenCompanionInventory(self.sendMessage);
        screenCompanionTeammates = new ScreenCompanionTeammates(self.sendMessage);
        screenCompanionSettings = new ScreenCompanionSettings(self.sendMessage);
        screenCompanionHelp = new ScreenCompanionHelp(self.sendMessage);
        screenCompanionDevTools = new ScreenCompanionDeveloperTools(self.sendMessage);
        screenCompanionDebugConsole = new ScreenCompanionDebugConsole();

        // initialize tab after all panel are initialized because it will trigger refreshView on the selected panel
        screenCompanionTabs = new ScreenCompanionTabs(onPanelSelected);

        
        // send ready message to C#
        this.sendMessage("ready", "-", ["-"]);
    }

    this.zoomForDevelopement = function()
    {
        containerDiv.style.zoom = 2;
        containerDiv.style.outline = "1px solid red";
    }

    this.addLogMessageHTML = function(htmlMsg)
    {
        screenCompanionDebugConsole.addLogMessageHTML(htmlMsg);
    }

    this.processMessage = function(jsonData)
    {
        try
        {

            //screenCompanionDebugConsole.addLogMessageHTML("<i>received from c#:</i> " + jsonData);
            var msg = JSON.parse(jsonData);
            if(msg["category"] == "system-check" && msg["action"] == "ping" && msg["parameters"][0] == "to JS")
            {
                self.sendMessage("system-check", "pong", ["from JS"]);
                self.addLogMessageHTML("JS: received a ping from c# sending back a pong");
            }
            else if(msg["category"] == "system-check" && msg["action"] == "pong" && msg["parameters"][0] == "from C#")
            {
                self.addLogMessageHTML("JS: received a pong from c# following our ping");
            }
            else if(msg["category"]=="console" && msg["action"]=="log")
            {
                var logMessageStr = msg["parameters"][0];
                logMessageStr = logMessageStr.replaceAll("[[CR]]", "\n");
                self.addLogMessageHTML(logMessageStr);
            }
            //else if(msg["category"] == "" && msg["action"] == "")
            else if(msg["category"] == "main-screen" && msg["action"] == "update-money")
            {
                moneyInfoDiv.textContent = "¥ " + msg["parameters"][0].toString().replace(/\B(?=(\d{3})+(?!\d))/g, " ");
            }
            else if(msg["category"] == "account" && msg["action"] == "update-data-all")
            {
                screenCompanionAccount.setDataAll(msg["parameters"]);
            }
            else if(msg["category"] == "account" && msg["action"] == "daily-capsules-count")
            {
                screenCompanionAccount.setDailyCapsulesCount(msg["parameters"]);
            }
            else if(msg["category"] == "settings" && msg["action"] == "update-data-all")
            {
                screenCompanionSettings.setDataAll(msg["parameters"]);
            }
            else if(msg["category"] == "teammates" && msg["action"] == "update-one-page")
            {
                screenCompanionTeammates.setPageData(msg["parameters"]);
            }
            else if(msg["category"] == "game" && msg["action"] == "connected-users")
            {
                screenCompanionGame.setConnectedUsers(msg["parameters"]);
            }
            else if(msg["category"] == "game" && msg["action"] == "user-connection")
            {
                screenCompanionGame.addConnectedUser(msg["parameters"]);
            }
            else if(msg["category"] == "game" && msg["action"] == "user-video-ready")
            {
                screenCompanionGame.setUserVideoIsReady(msg["parameters"]);
            }
            else if(msg["category"] == "game" && msg["action"] == "game-panel-html")
            {
                var panelDataStr = msg["parameters"][0];
                panelDataStr = panelDataStr.replaceAll("[[CR]]", "\n");
                //console.log("received GAME-PANEL-HTML");
                //console.log(panelDataStr);
                screenCompanionGame.setHTMLContent(panelDataStr);
            }
            else if(msg["category"] == "game-panel-plugin")
            {
                screenCompanionGame.forwardMessageToPlugin(msg);
            }
            
        }
        catch(e1)
        {
            console.log("ERROR: CompanionScreen cannot parse incoming message:\n" + e1.message + "\n" + e1.stack);
            console.log(jsonData);
        }
    }

    this.sendMessage = function(category, action, parameters)
    {
        sendRawMessageAction(JSON.stringify({"category": category, "action": action, "parameters": parameters}));
    }

    var drawClickAt = function(x,y){
        lastClickId++;
        var func = function(x){ return function(){ deleteClickBubble(x); } }(lastClickId); // make a localscope x variable which has the avlue of the global incremented variable
        setTimeout(func, 150);
        var d = document.createElement('DIV'); 
        d.setAttribute("id", "click_bubble_"+lastClickId);
        d.setAttribute('style', 'position: absolute; top:'+y+'px; left:'+x+'px; width:16px; height:16px; border:1px solid green; border-radius:16px; margin-top:-8px; margin-left:-8px;'); 
        document.body.appendChild(d); 
    }; 

    var deleteClickBubble = function(clickBubbleId)
    {
        var divToDelete = document.getElementById("click_bubble_"+clickBubbleId);
        divToDelete.parentNode.removeChild(divToDelete);
    }

    var onPanelSelected = function(panelName)
    {
        if(panelName == "Console")
        {
            screenCompanionDebugConsole.refreshView();
        }
    }

    init.call(self);
}