var Ankaa = Ankaa || {};
Ankaa.Client = Ankaa.Client || {};
Ankaa.Client.JSEngine = Ankaa.Client.JSEngine || {};

Ankaa.Client.JSEngine.BridgeClient = function(cache, sendMessageToEngineFunc)
{
    var self = this;

    /** @type {Ankaa.Client.JSEngine.StatesCache} */ 
    var states_cache = cache;

    var sendMessage = null;

    var asyncCallIdx = 0;

    var asyncResponsesManager = null;

    var init = function()
    {
        //console.log("Bridge to C# Initialized");

        sendMessage = sendMessageToEngineFunc;
        asyncResponsesManager = new Ankaa.Client.JSEngine.BridgeResponsesManager();

    }

    self.prepareWatch = function(goName)
    {
        var go = states_cache.get(goName);
        if(go == null)
        {
            var go = new Ankaa.Client.JSEngine.GameObject(this);
            go.name = goName;
            states_cache.add(goName, go);
        }
        sendMessage("js-bridge", "watch-go", [goName]); // we must inform the c# side that it should send us updates of this object
        return go;
    }

    self.listenCollisionEvent = function(goName)
    {
        var go = states_cache.get(goName);
        if(go == null)
        {
            var go = new Ankaa.Client.JSEngine.GameObject(this);
            go.name = goName;
            states_cache.add(goName, go);
        }
        sendMessage("js-bridge", "listen-collision", [goName]);
        return go;
    }

    self.destroyGameObject = function(goName)
    {
        states_cache.remove(goName);
        sendMessage("js-bridge", "destroy-go", [goName]);
    }

    self.callGameObjectMethod = function(method, params)
    {
        sendMessage("js-bridge", method, params);
    }

    self.testAsyncCallWithReponse = async function(message)
    {
        try
        {
            asyncCallIdx++;
            var msgId = "js-" + asyncCallIdx;
            var myPromise = asyncResponsesManager.createAwaitedResponseHandler(msgId);
            //console.log("JSBridge_JS: send async message to C# with Id : " + msgId + " with value: " + message);
            sendMessage("js-bridge", "test-async-call", [message], msgId);
            await myPromise.promise;
            //console.log("JSBridge_JS: returning for : " + msgId + " with response: " + JSON.stringify(myPromise.response));
            return myPromise.response;
        }catch(e1)
        {
            console.log(e1.message);
            console.log(e1.stack);
        }
    }

    self.processResponse = function(responseFor, parameters)
    {
        //console.log("JSBridge_JS: got a response for msg " + responseFor + " with values :" + JSON.stringify(parameters));
        asyncResponsesManager.releaseReponseHandler(responseFor, parameters);
    }

    self.updateWatches = function(serializedInfos)
    {
        //console.log(serializedInfos);
        var serializedObjects = serializedInfos.split('--');
        for(var objInfoIdx in serializedObjects)
        {
            var objInfo = serializedObjects[objInfoIdx];
            if(objInfo.length > 0)
            {
                var objInfoParts = objInfo.split('=');
                var objName = objInfoParts[0];
                //console.log(objName);
                var newPos = objInfoParts[1].split(';')
                var p = states_cache.get(objName).transform.position;
                p.x = newPos[0];
                p.y = newPos[1];
                p.z = newPos[2];
            }
        }
    }

    self.triggerCollisionEvents = function(serializedInfos)
    {
        var serializedObjects = serializedInfos.split('--');
        for(var objInfoIdx in serializedObjects)
        {
            var objInfo = serializedObjects[objInfoIdx];
            if(objInfo.length > 0)
            {
                try
                {
                    var objInfoParts = objInfo.split('=');
                    var collidedObjName = objInfoParts[0];
                    var colliderObjName = objInfoParts[1];
                    //states_cache.get(collidedObjName).triggerEvent('collision', {'collider': colliderObjName});
                    //console.log("A collision event has been propagated from c# side: " + collidedObjName + " with " + colliderObjName);
                    const collisionEvent = new Ankaa.Client.JSEngine.Event('collisionEntersEvent', {detail: { "sender": states_cache.get(collidedObjName), "collider": colliderObjName }});
                    states_cache.get(collidedObjName).dispatchEvent(collisionEvent);                
                }
                catch(e1)
                {
                    console.log(e1.message);
                    console.log(e1.stack);
                }
            }
        }
    }

    self.clearCaches = function()
    {
        //throw new Error("jsBridge.clearCaches is not implemented");
        states_cache.clearAll();
    }

    init.call(self);
}
