var Ankaa = Ankaa || {};
Ankaa.Client = Ankaa.Client || {};
Ankaa.Client.JSEngine = Ankaa.Client.JSEngine || {};

Ankaa.Client.JSEngine.SceneAPI = function(sendMessageToEngineFunc)
{
    if(Ankaa.Client.JSEngine.__SceneAPI_instance != null)
    {
        return Ankaa.Client.JSEngine.__SceneAPI_instance;
    }
        

    var self = this;

    /** @type {Ankaa.Client.JSEngine.StatesCache} */
    var cache = null;

    /** @type {Ankaa.Client.JSEngine.BridgeClient} */
    var bridge = null;

    var sendMessage = null;

    var messageReceivedFromTheatreScreenFunc = null;

    var init = function()
    {
        // singleton
        Ankaa.Client.JSEngine.__SceneAPI_instance = self;
        
        sendMessage = sendMessageToEngineFunc;

        cache = new Ankaa.Client.JSEngine.StatesCache();
        
        bridge = new Ankaa.Client.JSEngine.BridgeClient(cache, sendMessage);
        
        //console.log("SceneAPI singleton initialized");

    }

    self.setTheatreScreensMessageListenerFunc = function(funcMethod)
    {
        messageReceivedFromTheatreScreenFunc = funcMethod;
    }

    self.processMessage = function(msg)
    {

        //console.log("message to publish");
        //console.log(JSON.stringify(msg));
        if(msg["category"] == "js-bridge")
        {
            if(msg["action"] == "watched-go-updates")
                bridge.updateWatches(msg["parameters"][0]);
            else if(msg["action"] == "collision-events")
                bridge.triggerCollisionEvents(msg["parameters"][0]);
            else if(msg["action"] == "clear-caches")
                bridge.clearCaches();
            else if(msg["action"] == "response-for-async")
                bridge.processResponse(msg["responseFor"], msg["parameters"]);
        }
        else if(msg["category"] == "theatre-to-JSEngine")
        {
            if(messageReceivedFromTheatreScreenFunc != null)
            {
                try
                {
                    messageReceivedFromTheatreScreenFunc(msg["action"], msg["parameters"]);
                }catch(e1)
                {
                    console.log("EXCEPTION: exception in scene script: " + e1.message + "\n" + e1.stack );
                }
                
            }
        }
            
        
    }

    self.getInfo = function()
    {
        return "This is the main library used to access the objects in the 3D scene.";
    }

    /**
     * 
     * @param {string} goName 
     * @returns Ankaa.Client.JSEngine.GameObject
     */
    self.getGameObject = function(goName)
    {
        var go = new Ankaa.Client.JSEngine.GameObject(bridge);
        go.name = goName;
        return go;
    }

    self.watchGameObject = function(goName)
    {
        //console.log("Adding a watch to gameobject " + goName);
        var cachedGameObject = bridge.prepareWatch(goName);
        return cachedGameObject;
    }

    self.listenCollisionEvent = function(goName)
    {
        var cachedGameObject = bridge.listenCollisionEvent(goName);
        return cachedGameObject;
    }

    self.destroyGameObject = function(goName)
    {
        bridge.destroyGameObject(goName);
    }

    self.sendMessageToGamePanelPlugin = function(action, parameters)
    {
        sendMessage("game-panel-plugin", action, parameters);
    }

    self.testAsyncCallWithReponse = async function(message)
    {
        return await bridge.testAsyncCallWithReponse(message);
    }

    /**
     * 
     * @param {String} prefabName 
     * @param {Ankaa.Client.JSEngine.Vector3} position 
     */
    self.instanciatePrefab = function(prefabName, parentGoName, goName, position)
    {
        sendMessage("js-bridge", "instanciate-go", [prefabName, parentGoName, goName, position.toSerialized()]);
    }

    self.delayedCall = function(funcToCall, delayMs)
    {
        (async function() { 
            try
            {
                if(navigator.userAgent.indexOf('AnkaaHeadless')>-1)
                    await Task.Delay(delayMs); 
                else
                    await new Promise(r => setTimeout(r, delayMs));

                funcToCall();    
            }
            catch(e1)
            {
                console.log(e1.message);
                console.log(e1.stack);
            }
        })();
    }

    init.call(self);
}